@@ -75,8 +75,7 @@ static const char helpCommands2[] =
     "  --decrypt   "	"\tdecrypt data from XML document\n"
 #endif /* XMLSEC_NO_XMLENC */
 #ifndef XMLSEC_NO_XKMS
-    "  --xkis-server-locate  " "\tprocess data as XKMS/XKISS Locate request\n"
-    "  --xkis-server-validate" "\tprocess data as XKMS/XKISS Validate request\n"
+    "  --xkms-server-request ""\tprocess data as XKMS server request\n"
 #endif /* XMLSEC_NO_XKMS */
     ;
 
@@ -113,13 +112,9 @@ static const char helpDecrypt[] =
     "Usage: xmlsec decrypt [<options>] <file>\n"
     "Decrypts XML Encryption data in the <file>\n";
 
-static const char helpXkissServerLocate[] =  
-    "Usage: xmlsec xkiss-server-locate [<options>] <file>\n"
-    "Processes the <file> as XKMS/XKISS Locate request and outputs the response\n";
-
-static const char helpXkissServerValidate[] =  
-    "Usage: xmlsec xkms-server-validate [<options>] <file>\n"
-    "Processes the <file> as XKMS/XKISS Validate request and outputs the response\n";
+static const char helpXkmsServerRequest[] =  
+    "Usage: xmlsec xkms-server-request [<options>] <file>\n"
+    "Processes the <file> as XKMS server request and outputs the response\n";
 
 static const char helpListKeyData[] =     
     "Usage: xmlsec list-key-data\n"
@@ -638,6 +633,73 @@ static xmlSecAppCmdLineParam xmlDataParam = {
 };
 #endif /* XMLSEC_NO_XMLENC */
 
+/****************************************************************
+ *
+ * XKMS params
+ *
+ ***************************************************************/
+#ifndef XMLSEC_NO_XKMS
+static xmlSecAppCmdLineParam xkmsServiceParam = { 
+    xmlSecAppCmdLineTopicXkmsCommon,
+    "--xkms-service",
+    NULL,
+    "--xkms-service <uri>"
+    "\n\tsets XKMS \"Service\" <uri>",
+    xmlSecAppCmdLineParamTypeString,
+    xmlSecAppCmdLineParamFlagNone,
+    NULL
+};
+
+static xmlSecAppCmdLineParam xkmsFormatParam = { 
+    xmlSecAppCmdLineTopicXkmsCommon,
+    "--xkms-format",
+    NULL,
+    "--xkms-format <format>"
+    "\n\tsets the XKMS request/response format to one of the following values:"
+    "\n\t  \"plain\" (default), \"soap-1.1\" or \"soap-1.2\"",
+    xmlSecAppCmdLineParamTypeString,
+    xmlSecAppCmdLineParamFlagNone,
+    NULL
+};
+
+static xmlSecAppCmdLineParam xkmsStopUnknownResponseMechanismParam = { 
+    xmlSecAppCmdLineTopicXkmsCommon, /* todo: server */
+    "--xkms-stop-on-unknown-response-mechanism",
+    NULL,
+    "--xkms-stop-on-unknown-response-mechanism"
+    "\n\tstop processing XKMS server request if unknown ResponseMechanism"
+    "\n\tvalue was found",
+    xmlSecAppCmdLineParamTypeFlag,
+    xmlSecAppCmdLineParamFlagNone,
+    NULL
+};
+
+static xmlSecAppCmdLineParam xkmsStopUnknownRespondWithParam = { 
+    xmlSecAppCmdLineTopicXkmsCommon, /* todo: server */
+    "--xkms-stop-on-unknown-respond-with",
+    NULL,
+    "--xkms-stop-on-unknown-respond-with"
+    "\n\tstop processing XKMS server request if unknown RespondWith"
+    "\n\tvalue was found",
+    xmlSecAppCmdLineParamTypeFlag,
+    xmlSecAppCmdLineParamFlagNone,
+    NULL
+};
+
+static xmlSecAppCmdLineParam xkmsStopUnknownKeyUsageParam = { 
+    xmlSecAppCmdLineTopicXkmsCommon, /* todo: server */
+    "--xkms-stop-on-unknown-key-usage",
+    NULL,
+    "--xkms-stop-on-unknown-key-usage"
+    "\n\tstop processing XKMS server request if unknown KeyUsage"
+    "\n\tvalue was found",
+    xmlSecAppCmdLineParamTypeFlag,
+    xmlSecAppCmdLineParamFlagNone,
+    NULL
+};
+
+#endif /* XMLSEC_NO_XKMS */
+
 /****************************************************************
  *
  * X509 params
@@ -739,7 +801,7 @@ static xmlSecAppCmdLineParam depthParam = {
     NULL,    
     "--depth <number>"
     "\n\tmaximum certificates chain depth",
-    xmlSecAppCmdLineParamTypeTime,
+    xmlSecAppCmdLineParamTypeNumber,
     xmlSecAppCmdLineParamFlagNone,
     NULL
 };
@@ -772,6 +834,15 @@ static xmlSecAppCmdLineParamPtr parameters[] = {
     &xmlDataParam,
     &enabledCipherRefUrisParam,
 #endif /* XMLSEC_NO_XMLENC */
+
+    /* xkms params */
+#ifndef XMLSEC_NO_XKMS
+    &xkmsServiceParam,
+    &xkmsFormatParam,
+    &xkmsStopUnknownResponseMechanismParam,
+    &xkmsStopUnknownRespondWithParam,
+    &xkmsStopUnknownKeyUsageParam,
+#endif /* XMLSEC_NO_XKMS */
              
     /* common dsig and enc parameters */
     &sessionKeyParam,    
@@ -845,8 +916,7 @@ typedef enum {
     xmlSecAppCommandEncrypt,
     xmlSecAppCommandDecrypt,
     xmlSecAppCommandEncryptTmpl,
-    xmlSecAppCommandXkissServerLocate,
-    xmlSecAppCommandXkissServerValidate
+    xmlSecAppCommandXkmsServerRequest
 } xmlSecAppCommand;
 
 typedef struct _xmlSecAppXmlData				xmlSecAppXmlData,
@@ -895,10 +965,9 @@ static void			xmlSecAppPrintEncCtx		(xmlSecEncCtxPtr encCtx);
 #endif /* XMLSEC_NO_XMLENC */
 
 #ifndef XMLSEC_NO_XKMS
-static int			xmlSecAppXkissServerLocate	(const char* filename);
-static int			xmlSecAppXkissServerValidate	(const char* filename);
-static int			xmlSecAppPrepareXkissServerCtx	(xmlSecXkissServerCtxPtr xkissServerCtx);
-static void			xmlSecAppPrintXkissServerCtx	(xmlSecXkissServerCtxPtr xkissServerCtx);
+static int			xmlSecAppXkmsServerProcess	(const char* filename);
+static int			xmlSecAppPrepareXkmsServerCtx	(xmlSecXkmsServerCtxPtr xkmsServerCtx);
+static void			xmlSecAppPrintXkmsServerCtx	(xmlSecXkmsServerCtxPtr xkmsServerCtx);
 #endif /* XMLSEC_NO_XKMS */
 
 static void			xmlSecAppListKeyData		(void);
@@ -969,8 +1038,7 @@ int main(int argc, const char **argv) {
 	case xmlSecAppCommandVerify:
 	case xmlSecAppCommandEncrypt:
 	case xmlSecAppCommandDecrypt:
-	case xmlSecAppCommandXkissServerLocate:
-	case xmlSecAppCommandXkissServerValidate:
+	case xmlSecAppCommandXkmsServerRequest:
 	    if(pos >= argc) {
 		fprintf(stderr, "Error: <file> parameter is requried for this command\n");
 		xmlSecAppPrintUsage();
@@ -1081,18 +1149,10 @@ int main(int argc, const char **argv) {
 #endif /* XMLSEC_NO_XMLENC */
 
 #ifndef XMLSEC_NO_XKMS
-	case xmlSecAppCommandXkissServerLocate:
+	case xmlSecAppCommandXkmsServerRequest:
 	    for(i = pos; i < argc; ++i) {
-    	        if(xmlSecAppXkissServerLocate(argv[i]) < 0) {
-		    fprintf(stderr, "Error: failed to process XKISS Locate request from file \"%s\"\n", argv[i]);
-		    goto fail;
-		}
-	    }
-	    break;
-	case xmlSecAppCommandXkissServerValidate:
-	    for(i = pos; i < argc; ++i) {
-    	        if(xmlSecAppXkissServerValidate(argv[i]) < 0) {
-		    fprintf(stderr, "Error: failed to process XKISS Validate request from file \"%s\"\n", argv[i]);
+    	        if(xmlSecAppXkmsServerProcess(argv[i]) < 0) {
+		    fprintf(stderr, "Error: failed to process XKMS server request from file \"%s\"\n", argv[i]);
 		    goto fail;
 		}
 	    }
@@ -1788,9 +1848,12 @@ xmlSecAppPrintEncCtx(xmlSecEncCtxPtr encCtx) {
 
 #ifndef XMLSEC_NO_XKMS
 static int 
-xmlSecAppXkissServerLocate(const char* filename) {
+xmlSecAppXkmsServerProcess(const char* filename) {
     xmlSecAppXmlDataPtr data = NULL;
-    xmlSecXkissServerCtx xkissServerCtx;
+    xmlDocPtr doc = NULL;
+    xmlNodePtr result;
+    xmlSecXkmsServerCtx xkmsServerCtx;
+    xmlSecXkmsServerFormat format = xmlSecXkmsServerFormatPlain;
     clock_t start_time;
     int res = -1;
 
@@ -1798,32 +1861,53 @@ xmlSecAppXkissServerLocate(const char* filename) {
 	return(-1);
     }
 
-    if(xmlSecXkissServerCtxInitialize(&xkissServerCtx, gKeysMngr) < 0) {
-	fprintf(stderr, "Error: XKISS server context initialization failed\n");
+    if(xmlSecXkmsServerCtxInitialize(&xkmsServerCtx, gKeysMngr) < 0) {
+	fprintf(stderr, "Error: XKMS server context initialization failed\n");
 	return(-1);
     }
-    if(xmlSecAppPrepareXkissServerCtx(&xkissServerCtx) < 0) {
-	fprintf(stderr, "Error: XKISS server context preparation failed\n");
+    if(xmlSecAppPrepareXkmsServerCtx(&xkmsServerCtx) < 0) {
+	fprintf(stderr, "Error: XKMS server context preparation failed\n");
 	goto done;
     }
 
-    /* parse template and select start node */
-    data = xmlSecAppXmlDataCreate(filename, xmlSecNodeLocateRequest, xmlSecXkmsNs);
+    /* get the input format */
+    if(xmlSecAppCmdLineParamGetString(&xkmsFormatParam) != NULL) {
+	format = xmlSecXkmsServerFormatFromString(BAD_CAST xmlSecAppCmdLineParamGetString(&xkmsFormatParam));
+	if(format == xmlSecXkmsServerFormatUnknown) {
+	    fprintf(stderr, "Error: unknown format \"%s\"\n", 
+		    xmlSecAppCmdLineParamGetString(&xkmsFormatParam));
+	    return(-1);    
+	}
+    }
+
+    /* parse template and select start node, there are multiple options
+     * for start node thus we don't provide the default start node name */
+    data = xmlSecAppXmlDataCreate(filename, NULL, NULL);
     if(data == NULL) {
-	fprintf(stderr, "Error: failed to load template \"%s\"\n", filename);
+	fprintf(stderr, "Error: failed to load request from file \"%s\"\n", filename);
+	goto done;
+    }
+
+    /* prepare result document */
+    doc = xmlNewDoc(BAD_CAST "1.0");
+    if(doc == NULL) {
+	fprintf(stderr, "Error: failed to create doc\n");
 	goto done;
     }
 
     start_time = clock();          
-    if((xmlSecXkissServerCtxLocate(&xkissServerCtx, data->startNode) < 0) || (xkissServerCtx.result == NULL)) {
-	fprintf(stderr, "Error: failed to process locate request\n");
+    result = xmlSecXkmsServerCtxProcess(&xkmsServerCtx, data->startNode, format, doc);
+    if(result == NULL) {
+	fprintf(stderr, "Error: failed to process xkms server request\n");
 	goto done;
     }
     total_time += clock() - start_time;    
-    
+
+        
     /* print out result only once per execution */
+    xmlDocSetRootElement(doc, result);
     if(repeats <= 1) {
-	if(xmlSecAppWriteResult(xkissServerCtx.result, NULL) < 0) {
+	if(xmlSecAppWriteResult(doc, NULL) < 0) {
 	    goto done;
 	}
     }
@@ -1833,66 +1917,13 @@ xmlSecAppXkissServerLocate(const char* filename) {
 done:
     /* print debug info if requested */
     if(repeats <= 1) { 
-        xmlSecAppPrintXkissServerCtx(&xkissServerCtx);
+        xmlSecAppPrintXkmsServerCtx(&xkmsServerCtx);
     }
-    xmlSecXkissServerCtxFinalize(&xkissServerCtx);
+    xmlSecXkmsServerCtxFinalize(&xkmsServerCtx);
 
-    if(data != NULL) {
-	xmlSecAppXmlDataDestroy(data);
+    if(doc != NULL) {
+	xmlFreeDoc(doc);
     }
-    return(res);
-}
-
-static int 
-xmlSecAppXkissServerValidate(const char* filename) {
-    xmlSecAppXmlDataPtr data = NULL;
-    xmlSecXkissServerCtx xkissServerCtx;
-    clock_t start_time;
-    int res = -1;
-
-    if(filename == NULL) {
-	return(-1);
-    }
-
-    if(xmlSecXkissServerCtxInitialize(&xkissServerCtx, gKeysMngr) < 0) {
-	fprintf(stderr, "Error: XKISS server context initialization failed\n");
-	return(-1);
-    }
-    if(xmlSecAppPrepareXkissServerCtx(&xkissServerCtx) < 0) {
-	fprintf(stderr, "Error: XKISS server context preparation failed\n");
-	goto done;
-    }
-
-    /* parse template and select start node */
-    data = xmlSecAppXmlDataCreate(filename, xmlSecNodeValidateRequest, xmlSecXkmsNs);
-    if(data == NULL) {
-	fprintf(stderr, "Error: failed to load document \"%s\"\n", filename);
-	goto done;
-    }
-
-    start_time = clock();          
-    if((xmlSecXkissServerCtxValidate(&xkissServerCtx, data->startNode) < 0) || (xkissServerCtx.result == NULL)) {
-	fprintf(stderr, "Error: failed to process locate request\n");
-	goto done;
-    }
-    total_time += clock() - start_time;    
-    
-    /* print out result only once per execution */
-    if(repeats <= 1) {
-	if(xmlSecAppWriteResult(xkissServerCtx.result, NULL) < 0) {
-	    goto done;
-	}
-    }
-
-    res = 0;    
-
-done:
-    /* print debug info if requested */
-    if(repeats <= 1) { 
-        xmlSecAppPrintXkissServerCtx(&xkissServerCtx);
-    }
-    xmlSecXkissServerCtxFinalize(&xkissServerCtx);
-
     if(data != NULL) {
 	xmlSecAppXmlDataDestroy(data);
     }
@@ -1900,34 +1931,52 @@ done:
 }
 
 static int
-xmlSecAppPrepareXkissServerCtx(xmlSecXkissServerCtxPtr xkissServerCtx) {    
-    if(xkissServerCtx == NULL) {
-	fprintf(stderr, "Error: XKISS  context is null\n");
+xmlSecAppPrepareXkmsServerCtx(xmlSecXkmsServerCtxPtr xkmsServerCtx) {    
+    if(xkmsServerCtx == NULL) {
+	fprintf(stderr, "Error: XKMS context is null\n");
 	return(-1);
     }
 
     /* set key info params */
-    if(xmlSecAppPrepareKeyInfoReadCtx(&(xkissServerCtx->keyInfoReadCtx)) < 0) {
+    if(xmlSecAppPrepareKeyInfoReadCtx(&(xkmsServerCtx->keyInfoReadCtx)) < 0) {
 	fprintf(stderr, "Error: failed to prepare key info context\n");
 	return(-1);
     }
 
+    if(xmlSecAppCmdLineParamGetString(&xkmsServiceParam) != NULL) {
+	xkmsServerCtx->expectedService = xmlStrdup(BAD_CAST xmlSecAppCmdLineParamGetString(&xkmsServiceParam));
+	if(xkmsServerCtx->expectedService == NULL) {
+	    fprintf(stderr, "Error: failed to duplicate string \"%s\"\n", 
+		    xmlSecAppCmdLineParamGetString(&xkmsServiceParam));
+	    return(-1);    
+	}
+    }
+    
+    if(xmlSecAppCmdLineParamIsSet(&xkmsStopUnknownResponseMechanismParam)) {
+	xkmsServerCtx->flags |= XMLSEC_XKMS_SERVER_FLAGS_STOP_ON_UNKNOWN_RESPONSE_MECHANISM;
+    }
+    if(xmlSecAppCmdLineParamIsSet(&xkmsStopUnknownRespondWithParam)) {
+	xkmsServerCtx->flags |= XMLSEC_XKMS_SERVER_FLAGS_STOP_ON_UNKNOWN_RESPOND_WITH;
+    }
+    if(xmlSecAppCmdLineParamIsSet(&xkmsStopUnknownKeyUsageParam)) {
+	xkmsServerCtx->flags |= XMLSEC_XKMS_SERVER_FLAGS_STOP_ON_UNKNOWN_KEY_USAGE;
+    }
     return(0);
 }
 
 static void 
-xmlSecAppPrintXkissServerCtx(xmlSecXkissServerCtxPtr xkissServerCtx) {
-    if(xkissServerCtx == NULL) {
+xmlSecAppPrintXkmsServerCtx(xmlSecXkmsServerCtxPtr xkmsServerCtx) {
+    if(xkmsServerCtx == NULL) {
 	return;
     }
     
     /* print debug info if requested */
     if((print_debug != 0) || xmlSecAppCmdLineParamIsSet(&printDebugParam)) {
-	xmlSecXkissServerCtxDebugDump(xkissServerCtx, stdout);
+	xmlSecXkmsServerCtxDebugDump(xkmsServerCtx, stdout);
     }
     
     if(xmlSecAppCmdLineParamIsSet(&printXmlDebugParam)) {	   
-	xmlSecXkissServerCtxDebugXmlDump(xkissServerCtx, stdout);
+	xmlSecXkmsServerCtxDebugXmlDump(xkmsServerCtx, stdout);
     }
 }
 
@@ -2710,19 +2759,12 @@ xmlSecAppParseCommand(const char* cmd, xmlSecAppCmdLineParamTopic* cmdLineTopics
 #endif /* XMLSEC_NO_XMLENC */
 
 #ifndef XMLSEC_NO_XKMS
-    if((strcmp(cmd, "xkiss-server-locate") == 0) || (strcmp(cmd, "--xkiss-server-locate") == 0)) {
+    if(strcmp(cmd, "--xkms-server-request") == 0) {
 	(*cmdLineTopics) = xmlSecAppCmdLineTopicGeneral |
 			xmlSecAppCmdLineTopicXkmsCommon |
 			xmlSecAppCmdLineTopicKeysMngr |
 			xmlSecAppCmdLineTopicX509Certs;
-	return(xmlSecAppCommandXkissServerLocate);
-    } else 
-    if((strcmp(cmd, "xkiss-server-validate") == 0) || (strcmp(cmd, "--xkiss-server-validate") == 0)) {
-	(*cmdLineTopics) = xmlSecAppCmdLineTopicGeneral |
-			xmlSecAppCmdLineTopicXkmsCommon |
-			xmlSecAppCmdLineTopicKeysMngr |
-			xmlSecAppCmdLineTopicX509Certs;
-	return(xmlSecAppCommandXkissServerValidate);
+	return(xmlSecAppCommandXkmsServerRequest);
     } else
 #endif /* XMLSEC_NO_XKMS */
 
@@ -2769,11 +2811,8 @@ xmlSecAppPrintHelp(xmlSecAppCommand command, xmlSecAppCmdLineParamTopic topics)
     case xmlSecAppCommandEncryptTmpl:
 	fprintf(stdout, "%s\n", helpEncryptTmpl);
         break;
-    case xmlSecAppCommandXkissServerLocate:
-	fprintf(stdout, "%s\n", helpXkissServerLocate);
-        break;
-    case xmlSecAppCommandXkissServerValidate:
-	fprintf(stdout, "%s\n", helpXkissServerValidate);
+    case xmlSecAppCommandXkmsServerRequest:
+	fprintf(stdout, "%s\n", helpXkmsServerRequest);
         break;
     }
     if(topics != 0) {
