@@ -348,14 +348,29 @@ typedef int			(*xmlSecCryptoAppKeyCertLoadMemoryMethod)(xmlSecKeyPtr key,
  * @transformDes3CbcGetKlass:	the method to get pointer to Triple DES encryption transform.
  * @transformKWDes3GetKlass:	the method to get pointer to Triple DES key wrapper transform.
  * @transformDsaSha1GetKlass:	the method to get pointer to DSA-SHA1 signature transform.
- * @transformHmacSha1GetKlass:	the method to get pointer to HMAC-SHA1 transform.
- * @transformHmacRipemd160GetKlass: the method to get pointer to HMAC-RIPEMD160 transform.
  * @transformHmacMd5GetKlass:	the method to get pointer to HMAC-MD5 transform.
+ * @transformHmacRipemd160GetKlass: the method to get pointer to HMAC-RIPEMD160 transform.
+ * @transformHmacSha1GetKlass:	the method to get pointer to HMAC-SHA1 transform.
+ * @transformHmacSha224GetKlass: the method to get pointer to HMAC-SHA224 transform.
+ * @transformHmacSha256GetKlass: the method to get pointer to HMAC-SHA256 transform.
+ * @transformHmacSha384GetKlass: the method to get pointer to HMAC-SHA384 transform.
+ * @transformHmacSha512GetKlass: the method to get pointer to HMAC-SHA512 transform.
+ * @transformMd5GetKlass:	the method to get pointer to MD5 digest transform.
  * @transformRipemd160GetKlass:	the method to get pointer to RIPEMD160 digest transform.
+ * @transformRsaMd5GetKlass:	the method to get pointer to RSA-MD5 signature transform.
+ * @transformRsaRipemd160GetKlass: the method to get pointer to RSA-RIPEMD160 signature transform.
  * @transformRsaSha1GetKlass:	the method to get pointer to RSA-SHA1 signature transform.
+ * @transformRsaSha224GetKlass:	the method to get pointer to RSA-SHA224 signature transform.
+ * @transformRsaSha256GetKlass:	the method to get pointer to RSA-SHA256 signature transform.
+ * @transformRsaSha384GetKlass:	the method to get pointer to RSA-SHA384 signature transform.
+ * @transformRsaSha512GetKlass:	the method to get pointer to RSA-SHA512 signature transform.
  * @transformRsaPkcs1GetKlass:	the method to get pointer to RSA-PKCS1_5 key transport transform.
  * @transformRsaOaepGetKlass:	the method to get pointer to RSA-OAEP key transport transform.
  * @transformSha1GetKlass:	the method to get pointer to SHA1 digest transform.
+ * @transformSha224GetKlass:	the method to get pointer to SHA224 digest transform.
+ * @transformSha256GetKlass:	the method to get pointer to SHA256 digest transform.
+ * @transformSha384GetKlass:	the method to get pointer to SHA384 digest transform.
+ * @transformSha512GetKlass:	the method to get pointer to SHA512 digest transform.
  * @cryptoAppInit:		the default crypto engine initialization method.
  * @cryptoAppShutdown:		the default crypto engine shutdown method.
  * @cryptoAppDefaultKeysMngrInit: 	the default keys manager init method.
@@ -410,15 +425,30 @@ struct _xmlSecCryptoDLFunctions {
     xmlSecCryptoTransformGetKlassMethod		 transformDes3CbcGetKlass;
     xmlSecCryptoTransformGetKlassMethod		 transformKWDes3GetKlass;
     xmlSecCryptoTransformGetKlassMethod		 transformDsaSha1GetKlass;
-    xmlSecCryptoTransformGetKlassMethod		 transformHmacSha1GetKlass;
-    xmlSecCryptoTransformGetKlassMethod		 transformHmacRipemd160GetKlass;
     xmlSecCryptoTransformGetKlassMethod		 transformHmacMd5GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformHmacRipemd160GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformHmacSha1GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformHmacSha224GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformHmacSha256GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformHmacSha384GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformHmacSha512GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformMd5GetKlass;
     xmlSecCryptoTransformGetKlassMethod		 transformRipemd160GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformRsaMd5GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformRsaRipemd160GetKlass;
     xmlSecCryptoTransformGetKlassMethod		 transformRsaSha1GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformRsaSha224GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformRsaSha256GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformRsaSha384GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformRsaSha512GetKlass;
     xmlSecCryptoTransformGetKlassMethod		 transformRsaPkcs1GetKlass;
     xmlSecCryptoTransformGetKlassMethod		 transformRsaOaepGetKlass;
     xmlSecCryptoTransformGetKlassMethod		 transformSha1GetKlass;
-     
+    xmlSecCryptoTransformGetKlassMethod		 transformSha224GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformSha256GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformSha384GetKlass;
+    xmlSecCryptoTransformGetKlassMethod		 transformSha512GetKlass;
+    
     /**
      * High level routines form xmlsec command line utility
      */ 
@@ -439,34 +469,9 @@ struct _xmlSecCryptoDLFunctions {
     void*					 cryptoAppDefaultPwdCallback;
 };
 
-#ifdef XMLSEC_LIBXML_260
+#include <libxml/xmlstring.h>
 #define xmlSecStrPrintf 	xmlStrPrintf
-#else  /* XMLSEC_LIBXML_260 */
-#ifdef HAVE_SNPRINTF
-#define xmlSecStrPrintf 	snprintf
-#else  /* HAVE_SNPRINTF */
-#ifdef _MSC_VER
-#define xmlSecStrPrintf 	_snprintf
-#else  /* _MSC_VER */
-#error No snprintf function found
-#endif /* _MSC_VER */
-#endif /* HAVE_SNPRINTF */
-#endif /* XMLSEC_LIBXML_260 */
-
-#ifdef XMLSEC_LIBXML_260
 #define xmlSecStrVPrintf 	xmlStrVPrintf
-#else  /* XMLSEC_LIBXML_260 */
-#ifdef HAVE_VSNPRINTF
-#define xmlSecStrVPrintf 	vsnprintf
-#else  /* HAVE_VSNPRINTF */
-#ifdef _MSC_VER
-#define xmlSecStrVPrintf 	_vsnprintf
-#else  /* _MSC_VER */
-#error No vsnprintf function found
-#endif /* _MSC_VER */
-#endif /* HAVE_VSNPRINTF */
-#endif /* XMLSEC_LIBXML_260 */
-
 
 #ifdef __cplusplus
 }
