@@ -6,7 +6,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  * 
- * Copyrigth (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
+ * Copyright (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
  */
 #include "globals.h"
 
@@ -144,7 +144,7 @@ xmlSecTransformC14NNodeRead(xmlSecTransformPtr transform, xmlNodePtr node, xmlSe
     
         /* the list of namespaces is space separated */
 	for(p = n = list; ((p != NULL) && ((*p) != '\0')); p = n) {
-	    n = (xmlChar*)strchr(p, ' ');
+	    n = (xmlChar*)xmlStrchr(p, ' ');
 	    if(n != NULL) {
 	        *(n++) = '\0';
 	    }	
