@@ -22,6 +22,8 @@
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/keys.h>
 #include <xmlsec/transforms.h>
+#include <xmlsec/xmltree.h>
+#include <xmlsec/private.h>
 #include <xmlsec/errors.h>
 
 #include <xmlsec/openssl/app.h>
@@ -31,12 +33,8 @@
 
 static int 		xmlSecOpenSSLAppLoadRANDFile		(const char *file);
 static int 		xmlSecOpenSSLAppSaveRANDFile		(const char *file);
-static int		xmlSecOpenSSLDefaultPasswordCallback	(char *buf, int bufsiz, int verify, void *userdata);
-
-
-#if defined(_MSC_VER)
-#define snprintf _snprintf
-#endif
+static int		xmlSecOpenSSLDefaultPasswordCallback(char *buf, int bufsiz, int verify, void *userdata);
+static int      xmlSecOpenSSLDummyPasswordCallback  (char *buf, int bufsize, int verify, void *userdata);
 
 /**
  * xmlSecOpenSSLAppInit:
@@ -62,7 +60,7 @@ xmlSecOpenSSLAppInit(const char* config) {
 	return(-1);
     }
     
-    if((config != NULL) && (xmlSecOpenSSLSetDefaultTrustedCertsFolder(config) < 0)) {
+    if((config != NULL) && (xmlSecOpenSSLSetDefaultTrustedCertsFolder(BAD_CAST config) < 0)) {
 	xmlSecError(XMLSEC_ERRORS_HERE,
 		    NULL,
 		    "xmlSecOpenSSLSetDefaultTrustedCertsFolder",
@@ -205,6 +203,7 @@ xmlSecOpenSSLAppKeyLoadMemory(const xmlSecByte* data, xmlSecSize dataSize,
     return(key);
 }
 
+
 /**
  * xmlSecOpenSSLAppKeyLoadBIO:
  * @bio:		the key BIO.
@@ -233,11 +232,13 @@ xmlSecOpenSSLAppKeyLoadBIO(BIO* bio, xmlSecKeyDataFormat format,
     switch(format) {
     case xmlSecKeyDataFormatPem:
         /* try to read private key first */    
-	pKey = PEM_read_bio_PrivateKey(bio, NULL, (pem_password_cb*)pwdCallback, (void*)pwd);
+	    pKey = PEM_read_bio_PrivateKey(bio, NULL, 
+            (pwd != NULL) ? xmlSecOpenSSLDummyPasswordCallback : (pem_password_cb*)pwdCallback, 
+            (pwd != NULL) ? pwd : pwdCallbackCtx);
         if(pKey == NULL) {
     	    /* go to start of the file and try to read public key */
 	    BIO_reset(bio); 
-	    pKey = PEM_read_bio_PUBKEY(bio, NULL, (pem_password_cb*)pwdCallback, (void*)pwd);
+	    pKey = PEM_read_bio_PUBKEY(bio, NULL, (pem_password_cb*)pwdCallback, pwdCallbackCtx);
 	    if(pKey == NULL) {
 		xmlSecError(XMLSEC_ERRORS_HERE,
 			    NULL,
@@ -267,7 +268,7 @@ xmlSecOpenSSLAppKeyLoadBIO(BIO* bio, xmlSecKeyDataFormat format,
 	break;
     case xmlSecKeyDataFormatPkcs8Pem:
         /* try to read private key first */    
-	pKey = PEM_read_bio_PrivateKey(bio, NULL, (pem_password_cb*)pwdCallback, (void*)pwd);
+	pKey = PEM_read_bio_PrivateKey(bio, NULL, (pem_password_cb*)pwdCallback, pwdCallbackCtx);
         if(pKey == NULL) {
 	    xmlSecError(XMLSEC_ERRORS_HERE,
 			NULL,
@@ -279,7 +280,7 @@ xmlSecOpenSSLAppKeyLoadBIO(BIO* bio, xmlSecKeyDataFormat format,
 	break;
     case xmlSecKeyDataFormatPkcs8Der:
         /* try to read private key first */    
-	pKey = d2i_PKCS8PrivateKey_bio(bio, NULL, (pem_password_cb*)pwdCallback, (void*)pwd);
+	pKey = d2i_PKCS8PrivateKey_bio(bio, NULL, (pem_password_cb*)pwdCallback, pwdCallbackCtx);
         if(pKey == NULL) {
 	    xmlSecError(XMLSEC_ERRORS_HERE,
 			NULL,
@@ -289,6 +290,7 @@ xmlSecOpenSSLAppKeyLoadBIO(BIO* bio, xmlSecKeyDataFormat format,
 	    return(NULL);
 	}
 	break;
+#ifndef XMLSEC_NO_X509
     case xmlSecKeyDataFormatPkcs12:
 	key = xmlSecOpenSSLAppPkcs12LoadBIO(bio, pwd, pwdCallback, pwdCallbackCtx);
         if(key == NULL) {
@@ -301,7 +303,6 @@ xmlSecOpenSSLAppKeyLoadBIO(BIO* bio, xmlSecKeyDataFormat format,
 	}
 	return(key);
 	
-#ifndef XMLSEC_NO_X509
     case xmlSecKeyDataFormatCertPem:
     case xmlSecKeyDataFormatCertDer: 
 	key = xmlSecOpenSSLAppKeyFromCertLoadBIO(bio, format);
@@ -1438,7 +1439,7 @@ static int
 xmlSecOpenSSLDefaultPasswordCallback(char *buf, int bufsize, int verify, void *userdata) {
     char* filename = (char*)userdata;
     char* buf2;
-    char prompt[2048];
+    xmlChar prompt[2048];
     int i, ret;
         
     xmlSecAssert2(buf != NULL, -1);
@@ -1446,11 +1447,11 @@ xmlSecOpenSSLDefaultPasswordCallback(char *buf, int bufsize, int verify, void *u
     /* try 3 times */
     for(i = 0; i < 3; i++) {
         if(filename != NULL) {
-    	    snprintf(prompt, sizeof(prompt), "Enter password for \"%s\" file: ", filename); 
+    	    xmlSecStrPrintf(prompt, sizeof(prompt), BAD_CAST "Enter password for \"%s\" file: ", filename); 
 	} else {
-	    snprintf(prompt, sizeof(prompt), "Enter password: "); 
+	    xmlSecStrPrintf(prompt, sizeof(prompt), BAD_CAST "Enter password: "); 
         }
-	ret = EVP_read_pw_string(buf, bufsize, prompt, 0);
+	ret = EVP_read_pw_string(buf, bufsize, (char*)prompt, 0);
         if(ret != 0) {
     	    xmlSecError(XMLSEC_ERRORS_HERE,
 			NULL,
@@ -1462,13 +1463,13 @@ xmlSecOpenSSLDefaultPasswordCallback(char *buf, int bufsize, int verify, void *u
     
 	/* if we don't need to verify password then we are done */
         if(verify == 0) {
-	    return(0);
+	    return(strlen(buf));
         }
 
 	if(filename != NULL) {
-	    snprintf(prompt, sizeof(prompt), "Enter password for \"%s\" file again: ", filename); 
+	    xmlSecStrPrintf(prompt, sizeof(prompt), BAD_CAST "Enter password for \"%s\" file again: ", filename); 
 	} else {
-	    snprintf(prompt, sizeof(prompt), "Enter password again: "); 
+	    xmlSecStrPrintf(prompt, sizeof(prompt), BAD_CAST "Enter password again: "); 
 	}
 
 	buf2 = (char*)xmlMalloc(bufsize);
@@ -1480,7 +1481,7 @@ xmlSecOpenSSLDefaultPasswordCallback(char *buf, int bufsize, int verify, void *u
 			"size=%d", bufsize);
 	    return(-1);
 	}
-	ret = EVP_read_pw_string(buf2, bufsize, prompt, 0);
+	ret = EVP_read_pw_string(buf2, bufsize, (char*)prompt, 0);
 	if(ret != 0) {
 	    xmlSecError(XMLSEC_ERRORS_HERE,
 			NULL,
@@ -1495,8 +1496,8 @@ xmlSecOpenSSLDefaultPasswordCallback(char *buf, int bufsize, int verify, void *u
 	/* check if passwords match */
 	if(strcmp(buf, buf2) == 0) {
 	    memset(buf2, 0, bufsize);
-    	    xmlFree(buf2);
-	    return(-1);	    
+    	xmlFree(buf2);
+	    return(strlen(buf));	    
 	}
 	
 	/* try again */
@@ -1507,3 +1508,15 @@ xmlSecOpenSSLDefaultPasswordCallback(char *buf, int bufsize, int verify, void *u
     return(-1);
 }
 
+static int
+xmlSecOpenSSLDummyPasswordCallback(char *buf, int bufsize, int verify, void *userdata) {
+    char* password = (char*)userdata;
+    
+    if((password == NULL) || (strlen(password) + 1 > bufsize)) {
+        return(-1);
+    }
+    
+    strcpy(buf, password);
+    return (strlen(buf));
+}
+
