@@ -935,6 +935,15 @@ xmlSecOpenSSLKeyDataDsaGetType(xmlSecKeyDataPtr data) {
        
         if(dsa->priv_key != NULL) {
 	    return(xmlSecKeyDataTypePrivate | xmlSecKeyDataTypePublic);
+	} else if(dsa->engine != NULL) {
+	    /**
+	     * !!! HACK !!! Also see RSA key
+	     *
+	     * We assume here that engine *always* has private key.
+	     * This might be incorrect but it seems that there is no
+	     * way to ask engine if given key is private or not.
+	     */
+	    return(xmlSecKeyDataTypePrivate | xmlSecKeyDataTypePublic);
 	} else {
 	    return(xmlSecKeyDataTypePublic);
 	}
@@ -1498,6 +1507,15 @@ xmlSecOpenSSLKeyDataRsaGetType(xmlSecKeyDataPtr data) {
     if((rsa != NULL) && (rsa->n != NULL) && (rsa->e != NULL)) {
 	if(rsa->d != NULL) {
 	    return(xmlSecKeyDataTypePrivate | xmlSecKeyDataTypePublic);
+	} else if(rsa->engine != NULL) {
+	    /**
+	     * !!! HACK !!! Also see DSA key
+	     *
+	     * We assume here that engine *always* has private key.
+	     * This might be incorrect but it seems that there is no
+	     * way to ask engine if given key is private or not.
+	     */
+	    return(xmlSecKeyDataTypePrivate | xmlSecKeyDataTypePublic);
 	} else {
 	    return(xmlSecKeyDataTypePublic);
 	}
