@@ -183,7 +183,7 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
 
     if(parent->children == NULL) {
         /* TODO: add indents */
-	text = xmlNewText(BAD_CAST "\n"); 
+	text = xmlNewText(xmlSecStringCR); 
         if(text == NULL) {	
 	    xmlSecError(XMLSEC_ERRORS_HERE,
 			NULL,
@@ -212,12 +212,12 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
 	nsPtr = xmlSearchNs(cur->doc, cur, NULL);
 	if((nsPtr == NULL) || !xmlStrEqual(nsPtr->href, ns)) {
 	    nsPtr = xmlNewNs(cur, ns, NULL);
-	    xmlSetNs(cur, nsPtr);
 	}
+	xmlSetNs(cur, nsPtr);
     }
     
     /* TODO: add indents */
-    text = xmlNewText(BAD_CAST "\n"); 
+    text = xmlNewText(xmlSecStringCR); 
     if(text == NULL) {	
 	xmlSecError(XMLSEC_ERRORS_HERE,
 		    NULL,
@@ -249,7 +249,7 @@ xmlSecAddChildNode(xmlNodePtr parent, xmlNodePtr child) {
 
     if(parent->children == NULL) {
         /* TODO: add indents */
-	text = xmlNewText(BAD_CAST "\n"); 
+	text = xmlNewText(xmlSecStringCR); 
         if(text == NULL) {	
 	    xmlSecError(XMLSEC_ERRORS_HERE,
 			NULL,
@@ -264,7 +264,7 @@ xmlSecAddChildNode(xmlNodePtr parent, xmlNodePtr child) {
     xmlAddChild(parent, child);
 
     /* TODO: add indents */
-    text = xmlNewText(BAD_CAST "\n"); 
+    text = xmlNewText(xmlSecStringCR); 
     if(text == NULL) {	
 	xmlSecError(XMLSEC_ERRORS_HERE,
 		    NULL,
@@ -314,12 +314,12 @@ xmlSecAddNextSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
 	nsPtr = xmlSearchNs(cur->doc, cur, NULL);
 	if((nsPtr == NULL) || !xmlStrEqual(nsPtr->href, ns)) {
 	    nsPtr = xmlNewNs(cur, ns, NULL);
-	    xmlSetNs(cur, nsPtr);
 	}
+	xmlSetNs(cur, nsPtr);
     }
 
     /* TODO: add indents */
-    text = xmlNewText(BAD_CAST "\n");
+    text = xmlNewText(xmlSecStringCR);
     if(text == NULL) {	
 	xmlSecError(XMLSEC_ERRORS_HERE,
 		    NULL,
@@ -369,12 +369,12 @@ xmlSecAddPrevSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
 	nsPtr = xmlSearchNs(cur->doc, cur, NULL);
 	if((nsPtr == NULL) || !xmlStrEqual(nsPtr->href, ns)) {
 	    nsPtr = xmlNewNs(cur, ns, NULL);
-	    xmlSetNs(cur, nsPtr);
 	}
+	xmlSetNs(cur, nsPtr);
     }
 
     /* TODO: add indents */
-    text = xmlNewText(BAD_CAST "\n");
+    text = xmlNewText(xmlSecStringCR);
     if(text == NULL) {	
 	xmlSecError(XMLSEC_ERRORS_HERE,
 		    NULL,
@@ -460,40 +460,15 @@ xmlSecReplaceNode(xmlNodePtr node, xmlNodePtr newNode) {
  */
 int
 xmlSecReplaceContent(xmlNodePtr node, xmlNodePtr newNode) {
-    xmlNodePtr dummy;
-    xmlNodePtr ptr;
-
     xmlSecAssert2(node != NULL, -1);
     xmlSecAssert2(newNode != NULL, -1);  
-	    
-    dummy = xmlNewDocNode(newNode->doc, NULL, BAD_CAST "dummy", NULL);
-    if(dummy == NULL) {
-	xmlSecError(XMLSEC_ERRORS_HERE,
-		    NULL,
-		    "xmlNewDocNode",
-		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "node=dummy");
-	return(-1);
-    }
-	    
-    if(newNode == xmlDocGetRootElement(newNode->doc)) {
-	ptr = xmlDocSetRootElement(newNode->doc, dummy);
-    } else {
-	ptr = xmlReplaceNode(newNode, dummy);
-    }
-    if(ptr == NULL) {
-	xmlSecError(XMLSEC_ERRORS_HERE,
-		    NULL,
-		    "xmlDocSetRootElement or xmlReplaceNode",
-		    XMLSEC_ERRORS_R_XML_FAILED,
-		    XMLSEC_ERRORS_NO_MESSAGE);
-	xmlFreeNode(dummy);
-	return(-1);
-    }
-	    
+
+    xmlUnlinkNode(newNode);
+    xmlSetTreeDoc(newNode, node->doc);
     xmlNodeSetContent(node, NULL);
-    xmlAddChild(node, ptr);
-    xmlSetTreeDoc(ptr, node->doc);
+    xmlAddChild(node, newNode);
+    xmlSetTreeDoc(newNode, node->doc);
+
     return(0);
 }
 
@@ -511,47 +486,33 @@ xmlSecReplaceContent(xmlNodePtr node, xmlNodePtr newNode) {
 int
 xmlSecReplaceNodeBuffer(xmlNodePtr node, 
 			const xmlSecByte *buffer, xmlSecSize size) {
-    static const char dummyPrefix[] = "<dummy>";
-    static const char dummyPostfix[] = "</dummy>";
-    xmlDocPtr doc;
-    xmlNodePtr ptr1, ptr2;
+    xmlNodePtr results = NULL;
+    xmlNodePtr next = NULL;
 
     xmlSecAssert2(node != NULL, -1);
-    
-    doc = xmlSecParseMemoryExt((xmlSecByte*)dummyPrefix, strlen(dummyPrefix),
-			       buffer, size,
-			       (xmlSecByte*)dummyPostfix, strlen(dummyPostfix));
-    if(doc == NULL){
-	xmlSecError(XMLSEC_ERRORS_HERE,
-		    NULL,
-		    "xmlSecParseMemoryExt",
-		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-		    XMLSEC_ERRORS_NO_MESSAGE);
-	return(-1);	    	
+    xmlSecAssert2(node->parent != NULL, -1);
+
+    /* parse buffer in the context of node's parent */
+    if(xmlParseInNodeContext(node->parent, (const char*)buffer, size, XML_PARSE_NODICT, &results) != XML_ERR_OK) {
+        xmlSecError(XMLSEC_ERRORS_HERE,
+                    NULL,
+                    "xmlParseInNodeContext",
+                    XMLSEC_ERRORS_R_XML_FAILED,
+                    "Failed to parse content");
+        return(-1);         
     }
-	    
-    ptr1 = xmlDocGetRootElement(doc);
-    if(ptr1 == NULL){
-	xmlSecError(XMLSEC_ERRORS_HERE,
-		    NULL,
-		    "xmlDocGetRootElement",
-		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "root is null");
-	xmlFreeDoc(doc);
-	return(-1);	    	
+
+    /* add new nodes */
+    while (results != NULL) {
+        next = results->next;
+        xmlAddPrevSibling(node, results);
+        results = next;
     }
-    
-    ptr1 = ptr1->children;
-    while(ptr1 != NULL) {
-	ptr2 = ptr1->next;
-	xmlUnlinkNode(ptr1);
-	xmlAddPrevSibling(node, ptr1);
-	ptr1 = ptr2;
-    }
-	    
+
+    /* remove old node */
     xmlUnlinkNode(node);
     xmlFreeNode(node);  
-    xmlFreeDoc(doc);
+
     return(0);
 }
 
@@ -656,6 +617,7 @@ xmlSecGenerateAndAddID(xmlNodePtr node, const xmlChar* attrName, const xmlChar*
 	        return(-1);	
 	    }
 	    
+	    xmlFree(id);
 	    return(0);
 	}
 	xmlFree(id);
@@ -748,7 +710,7 @@ xmlSecGenerateID(const xmlChar* prefix, xmlSecSize len) {
 	
 	tmpLen = xmlStrlen(prefix) + xmlStrlen(res) + 1;
 	tmp = xmlMalloc(tmpLen + 1);
-	if(res == NULL) {
+	if(tmp == NULL) {
 	    xmlSecError(XMLSEC_ERRORS_HERE,
 		        NULL,
 			"xmlMalloc",
@@ -758,7 +720,7 @@ xmlSecGenerateID(const xmlChar* prefix, xmlSecSize len) {
 	    return(NULL);
 	}
 	
-	xmlSecStrPrintf(res, tmpLen, BAD_CAST "%s%s", prefix, res);
+	xmlSecStrPrintf(tmp, tmpLen, BAD_CAST "%s%s", prefix, res);
 	xmlFree(res);
 	res = tmp;    
     } else {
