@@ -1,8 +1,15 @@
 #!/bin/sh 
 
+OS_ARCH=`uname -o`
+
+if [ "z$OS_ARCH" = "zCygwin" ] ; then
+	topfolder=`cygpath -wa $2`
+	xmlsec_app=`cygpath -a $3`
+else
+	topfolder=$2
+	xmlsec_app=$3
+fi
 crypto=$1
-topfolder=$2
-xmlsec_app=$3
 file_format=$4
 
 pub_key_format=$file_format
@@ -13,10 +20,15 @@ priv_key_format="p12"
 if [ "z$TMPFOLDER" = "z" ] ; then
     TMPFOLDER=/tmp
 fi
-
 timestamp=`date +%Y%m%d_%H%M%S` 
-tmpfile=$TMPFOLDER/testDSig.$timestamp-$$.tmp
-logfile=$TMPFOLDER/testDSig.$timestamp-$$.log
+if [ "z$OS_ARCH" = "zCygwin" ] ; then
+	tmpfile=`cygpath -wa $TMPFOLDER/testDSig.$timestamp-$$.tmp`
+	logfile=`cygpath -wa $TMPFOLDER/testDSig.$timestamp-$$.log`
+else
+	tmpfile=$TMPFOLDER/testDSig.$timestamp-$$.tmp
+	logfile=$TMPFOLDER/testDSig.$timestamp-$$.log
+fi
+
 script="$0"
 
 # prepate crypto config folder
@@ -105,6 +117,149 @@ echo "--- testDSig started for xmlsec-$crypto library ($timestamp)" >> $logfile
 echo "--- LD_LIBRARY_PATH=$LD_LIBRARY_PATH" >> $logfile
 
 
+execDSigTest "" "aleksey-xmldsig-01/enveloping-dsa-x509chain" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509" \
+    "$priv_key_option $topfolder/keys/dsakey.$priv_key_format --pwd secret" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509"
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-rsa-x509chain" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509" \
+    "$priv_key_option $topfolder/keys/rsakey.$priv_key_format --pwd secret" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509"
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-md5-hmac-md5" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-md5-hmac-md5-64" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-ripemd160-hmac-ripemd160" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-ripemd160-hmac-ripemd160-64" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/xpointer-hmac" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha1-hmac-sha1" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha1-hmac-sha1-64" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha224-hmac-sha224" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha224-hmac-sha224-64" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha256-hmac-sha256" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha256-hmac-sha256-64" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha384-hmac-sha384" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha384-hmac-sha384-64" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha512-hmac-sha512" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha512-hmac-sha512-64" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" \
+    "--hmackey $topfolder/keys/hmackey.bin" 
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-md5-rsa-md5" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509" \
+    "$priv_key_option $topfolder/keys/rsakey.$priv_key_format --pwd secret" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509"
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-ripemd160-rsa-ripemd160" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509" \
+    "$priv_key_option $topfolder/keys/rsakey.$priv_key_format --pwd secret" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509"
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha1-rsa-sha1" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509" \
+    "$priv_key_option $topfolder/keys/rsakey.$priv_key_format --pwd secret" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509"
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha224-rsa-sha224" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509" \
+    "$priv_key_option $topfolder/keys/rsakey.$priv_key_format --pwd secret" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509"
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha256-rsa-sha256" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509" \
+    "$priv_key_option $topfolder/keys/rsakey.$priv_key_format --pwd secret" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509"
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha384-rsa-sha384" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509" \
+    "$priv_key_option $topfolder/keys/largersakey.$priv_key_format --pwd secret" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509"
+
+execDSigTest "" "aleksey-xmldsig-01/enveloping-sha512-rsa-sha512" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509" \
+    "$priv_key_option $topfolder/keys/largersakey.$priv_key_format --pwd secret" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509"
+
+#
+# To generate expired cert run the following command
+# > xmlsec1 sign --pkcs12 tests/keys/expiredkey.p12 --pwd secret --output out.xml ./tests/aleksey-xmldsig-01/enveloping-expired-cert.tmpl
+#
+execDSigTest "" "aleksey-xmldsig-01/enveloping-expired-cert" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509 --verification-time 2005-07-10+10:00:00" 
+
+
+execDSigTest "" "aleksey-xmldsig-01/dtd-hmac-91" \
+    "--hmackey $topfolder/keys/hmackey.bin --dtd-file $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd" \
+    "--hmackey $topfolder/keys/hmackey.bin --dtd-file $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd" \
+    "--hmackey $topfolder/keys/hmackey.bin --dtd-file $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd"
+
+execDSigTest "" "aleksey-xmldsig-01/x509data-test" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format" \
+    "$priv_key_option tests/keys/rsakey.$priv_key_format --pwd secret" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format"
+
+execDSigTest "" "aleksey-xmldsig-01/x509data-sn-test" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --untrusted-$cert_format $topfolder/keys/ca2cert.$cert_format  --untrusted-$cert_format $topfolder/keys/rsacert.$cert_format --enabled-key-data x509" \
+    "$priv_key_option tests/keys/rsakey.$priv_key_format --pwd secret" \
+    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --untrusted-$cert_format $topfolder/keys/ca2cert.$cert_format  --untrusted-$cert_format $topfolder/keys/rsacert.$cert_format --enabled-key-data x509"
+
 execDSigTest "" "merlin-xmldsig-twenty-three/signature-enveloped-dsa" \
     " " \
     "$priv_key_option $topfolder/keys/dsakey.$priv_key_format --pwd secret" \
@@ -184,60 +339,22 @@ execDSigTest "" "merlin-xmlenc-five/encsig-ripemd160-hmac-ripemd160-kw-tripledes
     "--keys-file $topfolder/merlin-xmlenc-five/keys.xml" \
     "--session-key hmac-192 --keys-file $topfolder/merlin-xmlenc-five/keys.xml" \
     "--keys-file $topfolder/merlin-xmlenc-five/keys.xml" 
+execDSigTest "" "merlin-xmlenc-five/encsig-sha256-hmac-sha256-kw-aes128" \
+    "--keys-file $topfolder/merlin-xmlenc-five/keys.xml" 
+execDSigTest "" "merlin-xmlenc-five/encsig-sha384-hmac-sha384-kw-aes192" \
+    "--keys-file $topfolder/merlin-xmlenc-five/keys.xml" 
+execDSigTest "" "merlin-xmlenc-five/encsig-sha512-hmac-sha512-kw-aes256" \
+    "--keys-file $topfolder/merlin-xmlenc-five/keys.xml" 
+
+#merlin-xmlenc-five/encsig-hmac-sha256-rsa-1_5.xml
+#merlin-xmlenc-five/encsig-hmac-sha256-rsa-oaep-mgf1p.xml
+
     
 execDSigTest "" "merlin-exc-c14n-one/exc-signature" \
     " " \
     "$priv_key_option $topfolder/keys/dsakey.$priv_key_format --pwd secret" \
     " " 
     
-execDSigTest "" "aleksey-xmldsig-01/enveloping-dsa-x509chain" \
-    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509" \
-    "$priv_key_option $topfolder/keys/dsakey.$priv_key_format --pwd secret" \
-    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509"
-
-execDSigTest "" "aleksey-xmldsig-01/enveloping-rsa-x509chain" \
-    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509" \
-    "$priv_key_option $topfolder/keys/rsakey.$priv_key_format --pwd secret" \
-    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509"
-
-execDSigTest "" "aleksey-xmldsig-01/enveloping-hmac-ripemd160" \
-    "--hmackey $topfolder/keys/hmackey.bin" \
-    "--hmackey $topfolder/keys/hmackey.bin" \
-    "--hmackey $topfolder/keys/hmackey.bin" 
-
-execDSigTest "" "aleksey-xmldsig-01/enveloping-hmac-ripemd160-64" \
-    "--hmackey $topfolder/keys/hmackey.bin" \
-    "--hmackey $topfolder/keys/hmackey.bin" \
-    "--hmackey $topfolder/keys/hmackey.bin" 
-
-execDSigTest "" "aleksey-xmldsig-01/enveloping-hmac-md5" \
-    "--hmackey $topfolder/keys/hmackey.bin" \
-    "--hmackey $topfolder/keys/hmackey.bin" \
-    "--hmackey $topfolder/keys/hmackey.bin" 
-
-execDSigTest "" "aleksey-xmldsig-01/enveloping-hmac-md5-64" \
-    "--hmackey $topfolder/keys/hmackey.bin" \
-    "--hmackey $topfolder/keys/hmackey.bin" \
-    "--hmackey $topfolder/keys/hmackey.bin" 
-
-execDSigTest "" "aleksey-xmldsig-01/xpointer-hmac" \
-    "--hmackey $topfolder/keys/hmackey.bin" \
-    "--hmackey $topfolder/keys/hmackey.bin" \
-    "--hmackey $topfolder/keys/hmackey.bin" 
-
-execDSigTest "" "aleksey-xmldsig-01/enveloping-expired-cert" \
-    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format --enabled-key-data x509 --verification-time 2002-04-17+10:00:00" 
-
-execDSigTest "" "aleksey-xmldsig-01/dtd-hmac-91" \
-    "--hmackey $topfolder/keys/hmackey.bin --dtd-file $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd" \
-    "--hmackey $topfolder/keys/hmackey.bin --dtd-file $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd" \
-    "--hmackey $topfolder/keys/hmackey.bin --dtd-file $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd"
-
-execDSigTest "" "aleksey-xmldsig-01/x509data-test" \
-    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format" \
-    "$priv_key_option tests/keys/rsakey.$priv_key_format --pwd secret" \
-    "--trusted-$cert_format $topfolder/keys/cacert.$cert_format"
-
 execDSigTest "" "merlin-exc-c14n-one/exc-signature" \
     ""
     
@@ -249,6 +366,7 @@ execDSigTest "" "merlin-xpath-filter2-three/sign-xfdl" \
 
 execDSigTest "" "merlin-xpath-filter2-three/sign-spec" \
     ""
+
 execDSigTest "phaos-xmldsig-three" "signature-big" \
     "--pubkey-cert-$cert_format certs/rsa-cert.$cert_format" 
 
@@ -264,8 +382,8 @@ execDSigTest "phaos-xmldsig-three" "signature-dsa-enveloping" \
 execDSigTest "phaos-xmldsig-three" "signature-dsa-manifest" \
     "--trusted-$cert_format certs/dsa-ca-cert.$cert_format"
 
-# execDSigTest "phaos-xmldsig-three" "signature-hmac-md5-c14n-enveloping" \
-#    "--hmackey certs/hmackey.bin"
+execDSigTest "phaos-xmldsig-three" "signature-hmac-md5-c14n-enveloping" \
+    "--hmackey certs/hmackey.bin"
     
 execDSigTest "phaos-xmldsig-three" "signature-hmac-sha1-40-c14n-comments-detached" \
     "--hmackey certs/hmackey.bin"
