@@ -1,5 +1,7 @@
 #!/bin/sh 
 
+OS_ARCH=`uname -o`
+
 crypto=$1
 topfolder=$2
 xmlsec_app=$3
@@ -7,7 +9,6 @@ file_format=$4
 
 pub_key_format=$file_format
 cert_format=$file_format
-crypto_config=$topfolder
 priv_key_option="--pkcs12"
 priv_key_format="p12"
 
@@ -19,7 +20,11 @@ timestamp=`date +%Y%m%d_%H%M%S`
 tmpfile=$TMPFOLDER/testEnc.$timestamp-$$.tmp
 logfile=$TMPFOLDER/testEnc.$timestamp-$$.log
 script="$0"
-keysfile=$topfolder/keys.xml
+
+# prepate crypto config folder
+crypto_config=$TMPFOLDER/xmlsec-crypto-config
+keysfile=$crypto_config/keys.xml
+
 valgrind_suppression="--suppressions=$topfolder/openssl.supp --suppressions=$topfolder/nss.supp"
 valgrind_options="--leak-check=yes --show-reachable=yes --num-callers=32 -v"
 
@@ -58,10 +63,16 @@ execEncTest() {
     printf "    Decrypt existing document                            "
     rm -f $tmpfile
 
+    if [ "z$OS_ARCH" = "zCygwin" ] ; then
+	diff_param=-uw
+    else
+	diff_param=-u
+    fi
+
     echo "$xmlsec_app decrypt $xmlsec_params $2 $file.xml" >>  $logfile 
     $VALGRIND $xmlsec_app decrypt $xmlsec_params $2 $file.xml > $tmpfile 2>> $logfile
     if [ $? = 0 ]; then
-	diff $file.data $tmpfile >> $logfile 2>> $logfile
+	diff $diff_param $file.data $tmpfile >> $logfile 2>> $logfile
 	printRes $?
     else 
 	echo " Error"
@@ -80,7 +91,7 @@ execEncTest() {
 		echo "$xmlsec_app decrypt $xmlsec_params $4 --output $tmpfile.2 $tmpfile" >>  $logfile 
 	        $VALGRIND $xmlsec_app decrypt $xmlsec_params $4 --output $tmpfile.2 $tmpfile >> $logfile 2>> $logfile
 		if [ $? = 0 ]; then
-		    diff $file.data $tmpfile.2 >> $logfile 2>> $logfile
+		    diff $diff_param $file.data $tmpfile.2 >> $logfile 2>> $logfile
 		    printRes $?
     		else 
 		    echo " Error"
@@ -201,11 +212,6 @@ execEncTest "merlin-xmlenc-five/encrypt-element-aes256-cbc-retrieved-kw-aes256"
 #merlin-xmlenc-five/encrypt-data-tripledes-cbc-rsa-oaep-mgf1p-sha256.xml
 #merlin-xmlenc-five/encsig-hmac-sha256-dh.xml
 #merlin-xmlenc-five/encsig-hmac-sha256-kw-tripledes-dh.xml
-#merlin-xmlenc-five/encsig-hmac-sha256-rsa-1_5.xml
-#merlin-xmlenc-five/encsig-hmac-sha256-rsa-oaep-mgf1p.xml
-#merlin-xmlenc-five/encsig-sha256-hmac-sha256-kw-aes128.xml
-#merlin-xmlenc-five/encsig-sha384-hmac-sha384-kw-aes192.xml
-#merlin-xmlenc-five/encsig-sha512-hmac-sha512-kw-aes256.xml
 
 execEncTest "01-phaos-xmlenc-3/enc-element-3des-kt-rsa1_5" \
     "$priv_key_option $topfolder/01-phaos-xmlenc-3/rsa-priv-key.$priv_key_format --pwd secret" \
@@ -308,12 +314,12 @@ execEncTest "01-phaos-xmlenc-3/enc-text-aes128-kw-aes192" \
 # test dynamic encryption
 echo "Dynamic encryption template"
 printf "    Encrypt template                                     "
-echo "$xmlsec_app encrypt-tmpl $xmlsec_params --keys-file $topfolder/keys.xml --output $tmpfile" >> $logfile
-$VALGRIND $xmlsec_app encrypt-tmpl $xmlsec_params --keys-file $topfolder/keys.xml --output $tmpfile >> $logfile 2>> $logfile
+echo "$xmlsec_app encrypt-tmpl $xmlsec_params --keys-file $keysfile --output $tmpfile" >> $logfile
+$VALGRIND $xmlsec_app encrypt-tmpl $xmlsec_params --keys-file $keysfile --output $tmpfile >> $logfile 2>> $logfile
 printRes $?
 printf "    Decrypt document                                     "
-echo "$xmlsec_app decrypt $xmlsec_params $topfolder/keys.xml $tmpfile" >> $logfile
-$VALGRIND $xmlsec_app decrypt $xmlsec_params --keys-file $topfolder/keys.xml $tmpfile >> $logfile 2>> $logfile
+echo "$xmlsec_app decrypt $xmlsec_params $keysfile $tmpfile" >> $logfile
+$VALGRIND $xmlsec_app decrypt $xmlsec_params --keys-file $keysfile $tmpfile >> $logfile 2>> $logfile
 printRes $?
 
 
